@@ -31,19 +31,24 @@ using Beagle.Util;
 using Beagle.Daemon;
 
 namespace Beagle.Filters {
-
+	
+	[PropertyKeywordMapping (Keyword="speakingto",     PropertyName="fixme:speakingto",    IsKeyword=true)]
 	public class FilterImLog : Beagle.Daemon.Filter {
 
 		private ImLog log;
 
 		public FilterImLog ()
 		{
-			AddSupportedFlavor (FilterFlavor.NewFromMimeType (GaimLog.MimeType));
-			AddSupportedFlavor (FilterFlavor.NewFromMimeType (KopeteLog.MimeType));
 			SnippetMode = true;
 			OriginalIsText = true;
 		}
 
+		protected override void RegisterSupportedTypes ()
+		{
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType (GaimLog.MimeType));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType (KopeteLog.MimeType));
+		}
+
 		override protected void DoOpen (FileInfo file)
 		{
 			try {
@@ -75,7 +80,7 @@ namespace Beagle.Filters {
 		{
 			foreach (ImLog.Utterance utt in log.Utterances) {
 				AppendText (utt.Text);
-				AppendText (" ");
+				AppendWhiteSpace ();
 			}
 
 			Finished ();
