@@ -62,6 +62,10 @@ namespace Beagle.Filters {
     		}
 
 		public FilterRPM ()
+		{
+		}
+
+		protected override void RegisterSupportedTypes ()
 		{
 			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/x-rpm"));
 		}
@@ -71,7 +75,12 @@ namespace Beagle.Filters {
 			SafeProcess pc = new SafeProcess ();
 			pc.Arguments = new string [] { "rpm", "-qp", "--queryformat", "[%{*:xml}\n]", FileInfo.FullName };
 			pc.RedirectStandardOutput = true;
-			pc.RedirectStandardError = true;
+
+			// Runs inside the child process after fork() but before exec()
+			pc.ChildProcessSetup += delegate {
+				// Let rpm run for 90 seconds, max.
+				SystemPriorities.SetResourceLimit (SystemPriorities.Resource.Cpu, 90);
+			};
 			
 			try {
 				pc.Start ();
@@ -190,7 +199,7 @@ namespace Beagle.Filters {
 			}
 
 			//Logger.Log.Debug ("    Done reading values. Now at " + 
-			//	(reader.IsStartElement () ? "" : "/") + reader.Name);
+			//	(reader.IsStartElement () ? String.Empty : "/") + reader.Name);
 		}
 	}
 }
