@@ -47,6 +47,10 @@ namespace Beagle.IndexHelper {
 		private static DateTime last_activity;
 		private static Server server;
 
+		// Current state with filtering.
+		public static Uri CurrentUri;
+		public static Filter CurrentFilter;
+
 		[DllImport ("libc")]
 		extern static private int unsetenv (string name);
 
@@ -82,18 +86,38 @@ namespace Beagle.IndexHelper {
 					LogLevel.Debug,
 					run_by_hand || log_in_fg);
 
-			// Intentionally unset DISPLAY so that we can't connect
-			// to the X server and aren't influenced by it if it
-			// goes away.  It's important to do this before
-			// Application.InitCheck(), since that's what makes the
-			// connection.
-			//unsetenv ("DISPLAY");
-
-			SystemInformation.XssInit (false);
+			Log.Always ("Starting Index Helper process (version {0})", ExternalStringsHack.Version);
+			Log.Always ("Running on {0}", SystemInformation.MonoRuntimeVersion);
+			Log.Always ("Using sqlite version {0}", ExternalStringsHack.SqliteVersion);
+			Log.Always ("Extended attributes are {0}", ExtendedAttribute.Supported ? "supported" : "not supported");
+			Log.Always ("Command Line: {0}",
+				    Environment.CommandLine != null ? Environment.CommandLine : "(null)");
 
 			// Initialize GObject type system
 			g_type_init ();
 
+			// Set the IO priority to idle, nice ourselves, and set
+			// a batch scheduling policy so we that we play nice
+			// on the system
+			if (Environment.GetEnvironmentVariable ("BEAGLE_EXERCISE_THE_DOG") == null) {
+				SystemPriorities.ReduceIoPriority ();
+
+				int nice_to_set;
+				
+				// We set different nice values because the
+				// internal implementation of SCHED_BATCH
+				// unconditionally imposes a +5 penalty on
+				// processes, and we want to be at nice +17,
+				// because it has a nice timeslice.
+				if (SystemPriorities.SetSchedulerPolicyBatch ())
+					nice_to_set = 12;
+				else
+					nice_to_set = 17;
+
+				SystemPriorities.Renice (nice_to_set);
+			} else
+				Log.Always ("BEAGLE_EXERCISE_THE_DOG is set");
+
 			Server.Init ();
 
 			SetupSignalHandlers ();
@@ -104,7 +128,7 @@ namespace Beagle.IndexHelper {
 			Shutdown.RegisterMainLoop (main_loop);
 
 			// Start the server
-			Logger.Log.Debug ("Starting messaging server");
+			Log.Debug ("Starting messaging server");
 			bool server_has_been_started = false;
 			try {
 				server = new Server ("socket-helper");
@@ -115,20 +139,6 @@ namespace Beagle.IndexHelper {
 			}
 
 			if (server_has_been_started) {
-				// Set the IO priority to idle so we don't slow down the system
-				if (Environment.GetEnvironmentVariable ("BEAGLE_EXERCISE_THE_DOG") == null) {
-					IoPriority.ReduceIoPriority ();
-
-					int prio = Mono.Unix.Native.Syscall.nice (15);
-
-					if (prio < 0)
-						Log.Warn ("Unable to renice helper to +15");
-					else if (prio == 15)
-						Log.Debug ("Reniced helper to +15");
-					else
-						Log.Debug ("Helper was already niced to {0}, not renicing to +15", prio);
-				}
-				
 				// Start the monitor thread, which keeps an eye on memory usage and idle time.
 				ExceptionHandlingThread.Start (new ThreadStart (MemoryAndIdleMonitorWorker));
 
@@ -149,7 +159,7 @@ namespace Beagle.IndexHelper {
 					} catch (IOException) { }
 				}
 
-				Log.Info ("Index helper process shut down cleanly.");
+				Log.Always ("Index helper process shut down cleanly.");
 			}
 		}
 
@@ -251,6 +261,7 @@ namespace Beagle.IndexHelper {
 			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGINT, OurSignalHandler);
 			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGTERM, OurSignalHandler);
 			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGUSR1, OurSignalHandler);
+			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGUSR2, OurSignalHandler);
 
 			// Ignore SIGPIPE
 			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGPIPE, Mono.Unix.Native.Stdlib.SIG_IGN);
@@ -269,8 +280,11 @@ namespace Beagle.IndexHelper {
 			if (signal < 0)
 				return;
 
-			// Set shutdown flag to true so that other threads can stop initializing
-			if ((Mono.Unix.Native.Signum) signal != Mono.Unix.Native.Signum.SIGUSR1)
+			// SIGUSR1 and SIGUSR2 are informational signals.  For other
+			// signals that we handle, set the shutdown flag to true so
+			// that other threads can stop initializing
+			if ((Mono.Unix.Native.Signum) signal != Mono.Unix.Native.Signum.SIGUSR1 &&
+			    (Mono.Unix.Native.Signum) signal != Mono.Unix.Native.Signum.SIGUSR2)
 				Shutdown.ShutdownRequested = true;
 
 			// Do all signal handling work in the main loop and not in the signal handler.
@@ -279,16 +293,29 @@ namespace Beagle.IndexHelper {
 
 		private static void HandleSignal (int signal)
 		{
-			Logger.Log.Debug ("Handling signal {0} ({1})", signal, (Mono.Unix.Native.Signum) signal);
+			Log.Warn ("Handling signal {0} ({1})", signal, (Mono.Unix.Native.Signum) signal);
 
 			// If we get SIGUSR1, turn the debugging level up.
 			if ((Mono.Unix.Native.Signum) signal == Mono.Unix.Native.Signum.SIGUSR1) {
 				LogLevel old_level = Log.Level;
 				Log.Level = LogLevel.Debug;
 				Log.Debug ("Moving from log level {0} to Debug", old_level);
-				return;
 			}
 
+			string span = StringFu.TimeSpanToString (DateTime.Now - last_activity);
+
+			if (CurrentUri == null)
+				Log.Warn ("Filtering status ({0} ago): no document is currently being filtered.", span);
+			else if (CurrentFilter == null)
+				Log.Warn ("Filtering status ({0} ago): determining filter for {1}", span, CurrentUri);
+			else
+				Log.Warn ("Filtering status ({0} ago): filtering {1} with {2}", span, CurrentUri, CurrentFilter);
+
+			// Don't shut down on information signals (SIGUSR1 and SIGUSR2)
+			if ((Mono.Unix.Native.Signum) signal == Mono.Unix.Native.Signum.SIGUSR1 ||
+			    (Mono.Unix.Native.Signum) signal == Mono.Unix.Native.Signum.SIGUSR2)
+				return;
+
 			Logger.Log.Debug ("Initiating shutdown in response to signal.");
 			Shutdown.BeginShutdown ();
 		}
